;+
; NAME:
;	moment4
; PURPOSE: (one line)
;	Compute various statistical moments of the data.
; DESCRIPTION:
;	This routine computes the average, average deviation, standard
;	deviation, variance, skew and kurtosis of the input data.  The various
;	output quantities are always returned as floating point scalars.
;	The statistics are compute with no regard for the dimensionality of
;	the input data.
; CATEGORY:
;	Statistics
; CALLING SEQUENCE:
;	moment4,data,avg,avgdev,stddev,var,skew,kurt
; INPUTS:
;	data - Input data to be analyzed.
; OPTIONAL INPUT PARAMETERS:
;	None.
; KEYWORD PARAMETERS:
;	None.
; OUTPUTS:
;	avg    - Sample mean.
;	avgdev - Average deviation of the data from the mean.
;	stddev - Standard deviation of the data from the mean.
;	var    - Variance of the data from the mean.
;	skew   - Skewness, third statistical moment.
;	kurt   - Kurtosis, fourth statistical moment.
; COMMON BLOCKS:
;	None.
; SIDE EFFECTS:
;	None.
; RESTRICTIONS:
;	None.
; PROCEDURE:
;	Standard techniques, see section 13.1 in Numerical Recipies.
; MODIFICATION HISTORY:
;	Written by Marc W. Buie, Lowell Observatory, 1992 Jan 20
;  1997/09/06, MWB, changed name from moment to avoid name conflict.
;-
pro moment4,data,avg,avgdev,stddev,var,skew,kurt

n = n_elements(data)

if n le 1 then begin
   print,'Error, number of elements in input data must be at least 2.'
   return
endif

avg = mean(data-data[0])+data[0]

s = data - avg

avgdev = mean(abs(s))

p = s*s
var = total(p)/(n-1)
stddev = sqrt(var)

if var ne 0.0 then begin
   p = p * s
   skew = total(p) / (n*stddev^3)
   p = p * s
   kurt = total(p) / (n * var^2) - 3.0
endif else begin
   skew = 0.0
   kurt=0.0
end

return
end
